@@ -27,6 +27,9 @@
 --
 --
 -- $Log: class_definition.c,v $
+-- Revision 1.30  2009/01/31 12:29:12  jsedwards
+-- Moved functions to deal with various file class definitions from objectify.c.
+--
 -- Revision 1.29  2008/08/31 21:53:51  jsedwards
 -- Added an assert around calls to nwos_read_variable_sized_object_from_disk
 -- and nwos_read_object_from_disk because now they return false when they fail
@@ -672,3 +675,154 @@ void nwos_find_or_create_public_feature_definition(ObjRef* type_ref, char* label
     }
 }
 #endif
+
+
+/*****************************************************************************/
+/* Functions to deal with the various file class definitions                 */
+/*****************************************************************************/
+
+#ifndef USE_PREDEFINED_STRUCTS
+
+/* These should only be referenced by using the get_file_XXX_reference functions below! */
+static ObjRef public_file_references[3];
+static ObjRef file_references[3];
+
+static void get_file_references()
+{
+    static volatile bool done = false;
+    ObjRef class_ref;
+    EveryObject header;
+    int i;
+
+    /* can't do anything until everything is initialized */
+    if (nwos_ready && !done)
+    {
+	done = true;   /* in case we get called recursively */
+
+	/* we know that as of version 0027 there are 3 versions of the file class */
+	nwos_find_public_class_definition("FILE", &public_file_references[2]);
+
+	nwos_read_object_headers_from_disk(&public_file_references[2], &header);
+	assert(!is_void_reference(&header.object.prev_version));
+	assert(is_void_reference(&header.object.next_version));
+
+	copy_reference(&public_file_references[1], &header.object.prev_version);
+
+	nwos_read_object_headers_from_disk(&public_file_references[1], &header);
+	assert(!is_void_reference(&header.object.prev_version));
+	assert(is_same_object(&header.object.next_version, &public_file_references[2]));
+
+	copy_reference(&public_file_references[0], &header.object.prev_version);
+
+	nwos_read_object_headers_from_disk(&public_file_references[0], &header);
+	assert(is_void_reference(&header.object.prev_version));
+	assert(is_same_object(&header.object.next_version, &public_file_references[1]));
+
+	copy_reference(&file_references[0], &public_file_references[0]);
+	copy_reference(&file_references[1], &public_file_references[1]);
+	copy_reference(&file_references[2], &public_file_references[2]);
+
+#ifndef PUBLIC_MODE
+	if (nwos_find_private_class_definition("FILE", &class_ref))       /* are there any private files? */
+	{
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+
+	    assert(is_void_reference(&header.object.next_version));
+	    assert(!is_void_reference(&header.object.clone_of));
+
+	    for (i = 2; i >= 0; i--)
+	    {
+		if (is_same_object(&public_file_references[i], &header.object.clone_of))
+		{
+		    copy_reference(&file_references[i], &class_ref);
+		    break;
+		}
+	    }
+
+	    while (i > 0)
+	    {
+		assert(!is_void_reference(&header.object.prev_version));
+
+		copy_reference(&class_ref, &header.object.prev_version);
+
+		if (nwos_reference_type(&class_ref) == Public_Reference) break;
+
+		nwos_read_object_headers_from_disk(&class_ref, &header);
+
+		assert(is_same_object(&header.object.next_version, &file_references[i]));
+
+		i--;
+
+		assert(is_same_object(&public_file_references[i], &header.object.clone_of));
+
+		copy_reference(&file_references[i], &class_ref);
+	    }
+	}
+#endif
+    }
+
+#if 0
+    printf("public_file_reference[0]: %08x\n", nwos_ref_to_word(&public_file_references[0]));
+    printf("public_file_reference[1]: %08x\n", nwos_ref_to_word(&public_file_references[1]));
+    printf("public_file_reference[2]: %08x\n", nwos_ref_to_word(&public_file_references[2]));
+    printf("\n");
+    printf("file_reference[0]: %08x\n", nwos_ref_to_word(&file_references[0]));
+    printf("file_reference[1]: %08x\n", nwos_ref_to_word(&file_references[1]));
+    printf("file_reference[2]: %08x\n", nwos_ref_to_word(&file_references[2]));
+#endif
+}
+
+
+ObjRef* nwos_get_file_001_reference()
+{
+    if (is_void_reference(&file_references[0]))
+    {
+	get_file_references();
+    }
+
+    return &file_references[0];
+}
+
+
+ObjRef* nwos_get_file_002_reference()
+{
+    if (is_void_reference(&file_references[1]))
+    {
+	get_file_references();
+    }
+
+    return &file_references[1];
+}
+
+
+ObjRef* nwos_get_file_class_reference()
+{
+    if (is_void_reference(&file_references[2]))
+    {
+	get_file_references();
+    }
+#ifndef PUBLIC_MODE
+    /* if storage is writable check to see if an updated file class has been created since this was first initialized */
+    else if (nwos_disk_io_is_read_write() && nwos_reference_type(&file_references[2]) == Public_Reference)
+    {
+	ObjRef class_ref;
+	EveryObject header;
+
+	if (nwos_find_private_class_definition("FILE", &class_ref))       /* see if one has been created since */
+	{
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+
+	    if (is_same_object(&header.object.clone_of, &public_file_references[2]))
+	    {
+		copy_reference(&file_references[2], &class_ref);
+		printf("Updating file class reference 2: %08x\n", nwos_ref_to_word(&file_references[2]));
+	    }
+	}
+    }
+#endif
+
+    return &file_references[2];
+}
+#endif
+
+
