@@ -27,6 +27,39 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.114  2008/12/30 13:15:15  jsedwards
+-- Put the log entries back in for 1.109, 1.110, and 1.111 which I accidentally
+-- left out in yesterdays check in.  NO code changes.
+--
+-- Revision 1.113  2008/12/29 13:49:17  jsedwards
+-- Removed the changes made in revisions 1.109, 1.110, and 1.111 which were
+-- experimental and in retrospect should have been done in a branch.  This
+-- revision is identical to revision 1.108.2.3 from the alpha_29_5_branch.
+--
+-- Revision 1.112  2008/12/27 16:10:16  jsedwards
+-- Merged from alpha_29_5_branch back into main trunk.
+--
+-- Revision 1.108.2.3  2008/12/26 16:24:24  jsedwards
+-- Changed nwos_read_block so offset is set to -1 when it is declared and
+-- removed code that specifically set it to -1 in different cases.
+--
+-- Revision 1.108.2.2  2008/12/26 16:10:25  jsedwards
+-- Fix Bug #2354266 - Somtimes it aborts when entering wrong pass phrase.
+-- Added code to nwos_read_block so that it returns false if the block is
+-- empty (I.E. the bit isn't set in the bit map).
+--
+-- Revision 1.108.2.1  2008/12/23 11:03:15  jsedwards
+-- Changed to use strlcpy instead of strcpy.
+--
+-- Revision 1.111  2008/11/04 14:07:15  jsedwards
+-- EXPERIMENTAL - Does not work! - Added check_chunk_density function, incomplete.
+--
+-- Revision 1.110  2008/11/03 10:39:13  jsedwards
+-- EXPERIMENTAL - Does not work! - Deleted analyze_chunk_density, etc.
+--
+-- Revision 1.109  2008/11/02 14:00:58  jsedwards
+-- EXPERIMENTAL - Does not work! - Added analyze_chunk_density, etc.
+--
 -- Revision 1.108  2008/11/01 10:22:09  jsedwards
 -- Missed one change of RESERVED_PUBLIC_BLOCKS to MAXIMUM_PUBLIC_REFERENCE in
 -- previous check in.
@@ -605,7 +638,7 @@ void index_compressed_file()
 
     i = strlen(nwos_private_path);
     index_path = nwos_malloc(i+5);  /* 4 for .ndx and 1 for null */
-    strcpy(index_path, nwos_private_path);
+    strlcpy(index_path, nwos_private_path, i+5);
 
     if (index_path[i-4] == '.' && index_path[i-3] == 'o' && index_path[i-2] == 'b' && index_path[i-1] == 'j')
     {
@@ -2502,13 +2535,14 @@ void nwos_generate_new_public_id(ObjRef* ref)
 bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
 {
     char ref_str[64];
-    off_t offset;
+    off_t offset = -1;
     int i = 0;
     int lower;
     int upper;
     int file_desc;
     uint32 ref_hash;
     uint32 mid_hash;
+    uint32 hash;
 
     assert(!is_void_reference(ref));
 
@@ -2543,7 +2577,12 @@ bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
     else if (storage_index == NULL)     /* normal storage */
     {
 	file_desc = private_file_desc;
-	offset = ref_to_offset(ref);
+	hash = nwos_hash_ref(ref);
+
+	if (hash != 0 && nwos_test_bit_in_map(hash))
+	{
+	    offset = (off_t) hash << 8;
+	}
     }
     else
     {
@@ -2603,8 +2642,6 @@ bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
 	    {
 		fprintf(stderr, "  %d: %08x\n", i, nwos_ref_to_word(&storage_index[i]));
 	    }
-
-	    offset = (off_t) -1;
 	}
 	else
 	{
