@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2006-2008  QRW Software
+--                 Copyright (C) 2006-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,10 @@
 --
 --
 -- $Log: export_file.c,v $
+-- Revision 1.25  2009/01/26 16:25:53  jsedwards
+-- Changed boolean result to count the number of failures and report it at the
+-- end.  Also use the number of failures as the exit code instead of just 1.
+--
 -- Revision 1.24  2008/12/27 15:00:29  jsedwards
 -- Merged from alpha_29_5_branch back into main trunk.
 --
@@ -269,8 +273,8 @@ int main(int argc, char* argv[])
     ReferenceList* ref_list;
     int num_refs;
     int num_files = 0;
+    int num_failures = 0;
     bool match;
-    bool result = true;
 
 
     if (argc > 1 && *argv[1] == '-')
@@ -337,7 +341,7 @@ int main(int argc, char* argv[])
 		    }
 		    else
 		    {
-			result = false;
+			num_failures++;
 		    }
 
 		    match = true;
@@ -373,7 +377,7 @@ int main(int argc, char* argv[])
 			    }
 			    else
 			    {
-				result = false;
+				num_failures++;
 			    }
 
 			    match = true;
@@ -387,18 +391,19 @@ int main(int argc, char* argv[])
 	{
 	    printf("No file(s) matching '%s' found.\n", argv[i]);
 
-	    result = false;
+	    num_failures++;
 	}
     }
 
-    printf("Number of files: %d\n", num_files);
+    printf("Number of files expored: %d\n", num_files);
+    printf("Number of files failed: %d\n", num_failures);
 
     nwos_free_reference_list(ref_list);
     ref_list = NULL;
 
     nwos_terminate_objectify();
 
-    return !result;    /* return 0 means success */
+    return num_failures;    /* return 0 means success */
 }
 
 
