@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2006-2008  QRW Software
+--                 Copyright (C) 2006-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,9 @@
 --
 --
 -- $Log: list_files.c,v $
+-- Revision 1.37  2009/01/26 16:32:52  jsedwards
+-- Changed to use PATH_MAX for size of name buffer instead of 256.
+--
 -- Revision 1.36  2008/12/27 15:00:29  jsedwards
 -- Merged from alpha_29_5_branch back into main trunk.
 --
@@ -173,6 +176,7 @@
 */
 
 #include <fnmatch.h>
+#include <limits.h>    /* define PATH_MAX */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -317,7 +321,7 @@ int main(int argc, char* argv[])
     int j;
     int revision;
     int old_file_revisions = 0;
-    char name[256];
+    char name[PATH_MAX];
     bool do_md5 = false;
     bool do_sha1 = false;
     bool do_sha256 = false;
