@@ -27,6 +27,20 @@
 --
 --
 -- $Log: log_disc.c,v $
+-- Revision 1.40  2008/09/01 03:13:43  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.39  2008/08/31 21:53:53  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.38  2008/08/30 12:46:14  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
 -- Revision 1.37  2008/08/03 17:31:34  jsedwards
 -- Add --ignore-empty-directories and --ignore-empty-files options.
 --
@@ -191,11 +205,6 @@ static void get_input(char* descr, char* buffer, size_t size)
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_object_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     ObjRef ref;
     int i;
     char id[13];
@@ -278,19 +287,11 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
-
-    nwos_set_root_object(&root_object_ref);
+    nwos_initialize_objectify(READ_WRITE, DEFAULT_FILE);
 
     nwos_set_security_level(Security_Very_Low);
 
+
     if (!nwos_check_blocks_available(14))   /* 14 if new storage location is created, normally just 2 */
     {
 	fprintf(stderr, "Cannot create new storage location!\n");
@@ -375,7 +376,7 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
-    nwos_read_variable_sized_object_from_disk(&disc_list_ref, kludge, sizeof(kludge), &nwos_get_disc_list_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(&disc_list_ref, kludge, sizeof(kludge), &nwos_get_disc_list_object_size));
 
     memcpy(id, ptr_list_obj->id, sizeof(id) - 1);
     id[sizeof(id) - 1] = '\0';
@@ -434,7 +435,7 @@ int main(int argc, char* argv[])
 
 	for (i = 0; i < num_locations; i++)
 	{
-	    nwos_read_object_from_disk(&location_ref[i], &loc_obj, sizeof(loc_obj));
+	    assert(nwos_read_object_from_disk(&location_ref[i], &loc_obj, sizeof(loc_obj)));
 	    nwos_name_to_string(&loc_obj.name, name, sizeof(name));
 	    printf("   %d=%s\n", i + 1, name);
 	}
