@@ -27,6 +27,10 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.182  2009/01/31 12:30:20  jsedwards
+-- Moved functions to deal with various file class definitions to
+-- class_definition.c file.
+--
 -- Revision 1.181  2008/10/31 13:00:57  jsedwards
 -- Moved nwos_reference_type and is_valid_private_reference functions to
 -- disk_io.c and renamed the latter to nwos_is_private_reference.
@@ -1781,151 +1785,6 @@ bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t si
 }
 
 
-#ifndef USE_PREDEFINED_STRUCTS
-
-/* These should only be referenced by using the get_file_XXX_reference functions below! */
-static ObjRef public_file_references[3];
-static ObjRef file_references[3];
-
-static void get_file_references()
-{
-    static volatile bool done = false;
-    ObjRef class_ref;
-    EveryObject header;
-    int i;
-
-    /* can't do anything until everything is initialized */
-    if (nwos_ready && !done)
-    {
-	done = true;   /* in case we get called recursively */
-
-	/* we know that as of version 0027 there are 3 versions of the file class */
-	nwos_find_public_class_definition("FILE", &public_file_references[2]);
-
-	nwos_read_object_headers_from_disk(&public_file_references[2], &header);
-	assert(!is_void_reference(&header.object.prev_version));
-	assert(is_void_reference(&header.object.next_version));
-
-	copy_reference(&public_file_references[1], &header.object.prev_version);
-
-	nwos_read_object_headers_from_disk(&public_file_references[1], &header);
-	assert(!is_void_reference(&header.object.prev_version));
-	assert(is_same_object(&header.object.next_version, &public_file_references[2]));
-
-	copy_reference(&public_file_references[0], &header.object.prev_version);
-
-	nwos_read_object_headers_from_disk(&public_file_references[0], &header);
-	assert(is_void_reference(&header.object.prev_version));
-	assert(is_same_object(&header.object.next_version, &public_file_references[1]));
-
-	copy_reference(&file_references[0], &public_file_references[0]);
-	copy_reference(&file_references[1], &public_file_references[1]);
-	copy_reference(&file_references[2], &public_file_references[2]);
-
-#ifndef PUBLIC_MODE
-	if (nwos_find_private_class_definition("FILE", &class_ref))       /* are there any private files? */
-	{
-	    nwos_read_object_headers_from_disk(&class_ref, &header);
-
-	    assert(is_void_reference(&header.object.next_version));
-	    assert(!is_void_reference(&header.object.clone_of));
-
-	    for (i = 2; i >= 0; i--)
-	    {
-		if (is_same_object(&public_file_references[i], &header.object.clone_of))
-		{
-		    copy_reference(&file_references[i], &class_ref);
-		    break;
-		}
-	    }
-
-	    while (i > 0)
-	    {
-		assert(!is_void_reference(&header.object.prev_version));
-
-		copy_reference(&class_ref, &header.object.prev_version);
-
-		if (nwos_reference_type(&class_ref) == Public_Reference) break;
-
-		nwos_read_object_headers_from_disk(&class_ref, &header);
-
-		assert(is_same_object(&header.object.next_version, &file_references[i]));
-
-		i--;
-
-		assert(is_same_object(&public_file_references[i], &header.object.clone_of));
-
-		copy_reference(&file_references[i], &class_ref);
-	    }
-	}
-#endif
-    }
-
-#if 0
-    printf("public_file_reference[0]: %08x\n", nwos_ref_to_word(&public_file_references[0]));
-    printf("public_file_reference[1]: %08x\n", nwos_ref_to_word(&public_file_references[1]));
-    printf("public_file_reference[2]: %08x\n", nwos_ref_to_word(&public_file_references[2]));
-    printf("\n");
-    printf("file_reference[0]: %08x\n", nwos_ref_to_word(&file_references[0]));
-    printf("file_reference[1]: %08x\n", nwos_ref_to_word(&file_references[1]));
-    printf("file_reference[2]: %08x\n", nwos_ref_to_word(&file_references[2]));
-#endif
-}
-
-
-static ObjRef* get_file_001_reference()
-{
-    if (is_void_reference(&file_references[0]))
-    {
-	get_file_references();
-    }
-
-    return &file_references[0];
-}
-
-
-static ObjRef* get_file_002_reference()
-{
-    if (is_void_reference(&file_references[1]))
-    {
-	get_file_references();
-    }
-
-    return &file_references[1];
-}
-
-
-static ObjRef* get_file_class_reference()
-{
-    if (is_void_reference(&file_references[2]))
-    {
-	get_file_references();
-    }
-#ifndef PUBLIC_MODE
-    /* if storage is writable check to see if an updated file class has been created since this was first initialized */
-    else if (nwos_disk_io_is_read_write() && nwos_reference_type(&file_references[2]) == Public_Reference)
-    {
-	ObjRef class_ref;
-	EveryObject header;
-
-	if (nwos_find_private_class_definition("FILE", &class_ref))       /* see if one has been created since */
-	{
-	    nwos_read_object_headers_from_disk(&class_ref, &header);
-
-	    if (is_same_object(&header.object.clone_of, &public_file_references[2]))
-	    {
-		copy_reference(&file_references[2], &class_ref);
-		printf("Updating file class reference 2: %08x\n", nwos_ref_to_word(&file_references[2]));
-	    }
-	}
-    }
-#endif
-
-    return &file_references[2];
-}
-#endif
-
-
 #define SIZE_EXTRA_BLOCK_DATA (FILE_BLOCK_SIZE - sizeof(uint32) - sizeof(ObjRef) - sizeof(ObjRef))
 
 typedef struct
@@ -2040,7 +1899,7 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 #ifndef USE_PREDEFINED_STRUCTS
 #ifndef PUBLIC_MODE
     /* check to see if this is a file object (which has an older revision) */
-    if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_001_reference()))
+    if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, nwos_get_file_001_reference()))
     {
 	if (check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum))
 	{
@@ -2048,7 +1907,7 @@ bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 	    return true;
 	}
     }
-    else if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_002_reference()))
+    else if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, nwos_get_file_002_reference()))
     {
 	if (check_object_data(ref, (uint8*)object, sizeof(C_struct_File_002), header->common.data_chksum))
 	{
@@ -2445,13 +2304,13 @@ void nwos_get_object_class(ObjRef* obj, ObjRef* object_class)
     nwos_get_object_class_without_update(obj, object_class);
 
 #ifndef USE_PREDEFINED_STRUCTS
-    if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, get_file_001_reference()))
+    if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, nwos_get_file_001_reference()))
     {
-	copy_reference(object_class, get_file_class_reference());
+	copy_reference(object_class, nwos_get_file_class_reference());
     }
-    else if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, get_file_002_reference()))
+    else if (nwos_reference_type(obj) == Private_Reference && is_same_object(object_class, nwos_get_file_002_reference()))
     {
-	copy_reference(object_class, get_file_class_reference());
+	copy_reference(object_class, nwos_get_file_class_reference());
     }
 #endif
 }
