@@ -27,6 +27,13 @@
 --
 --
 -- $Log: objectify_private.h,v $
+-- Revision 1.80  2009/01/31 15:39:25  jsedwards
+-- Added NUM_FILE_CLASS_DEFINIIONS define.
+--
+-- Revision 1.79  2009/01/31 12:34:16  jsedwards
+-- Added external declarations for nwos_ready and nwos_get_file_XXX_reference
+-- functions.
+--
 -- Revision 1.78  2008/10/31 12:35:31  jsedwards
 -- Added MAXIMUM_PUBLIC_REFERENCE, MINIMUM_PRIVATE_REFERENCE,
 -- MINIMUM_TEMPORARY_REFERENCE, MAXIMUM_TEMPORARY_REFERENCE, and changed
@@ -494,6 +501,8 @@ extern ObjRef nwos_private_class_definition_class_ref;
 extern ObjRef nwos_reference_list_class_ref;
 extern ObjRef nwos_next_public_ref;
 
+extern bool nwos_ready;
+
 extern const char* nwos_get_public_objects_path();
 extern const char* nwos_get_private_objects_path();
 extern const char* nwos_get_log_file_path();
@@ -502,6 +511,11 @@ extern const char* nwos_get_backup_directory_path();
 extern void nwos_create_class_definition(char* name_of_class);
 extern void nwos_clone_class_definition(ObjRef* class_to_clone_ref, ObjRef* clone_ref);
 
+#define NUM_FILE_CLASS_DEFINITIONS  3
+
+extern ObjRef* nwos_get_file_001_reference();
+extern ObjRef* nwos_get_file_002_reference();
+extern ObjRef* nwos_get_file_class_reference();
 
 extern void* nwos_malloc(size_t size);
 extern void  nwos_free(void* ptr);
