@@ -27,6 +27,13 @@
 --
 --
 -- $Log: person.c,v $
+-- Revision 1.26  2008/12/27 15:00:29  jsedwards
+-- Merged from alpha_29_5_branch back into main trunk.
+--
+-- Revision 1.25.2.1  2008/12/24 14:37:45  jsedwards
+-- Fix Bug #2051304 - changed assert in list_persons to print a warning message
+-- and return instead.
+--
 -- Revision 1.25  2008/09/01 00:10:54  jsedwards
 -- Fix copyright year.  NO code changes.
 --
@@ -973,7 +980,11 @@ void nwos_list_persons()
     int i;
     char name[32];
 
-    assert(nwos_find_private_class_definition("PERSON", &person_class_ref));
+    if (!nwos_find_private_class_definition("PERSON", &person_class_ref))
+    {
+	printf("\nThere are no persons in the system!\n\n");
+	return;
+    }
 
     nwos_read_object_headers_from_disk(&person_class_ref, &class_ref_header);
 
