@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                    Copyright (C) 2008  QRW Software
+--                 Copyright (C) 2008-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,13 @@
 --
 --
 -- $Log: progress_bar.c,v $
+-- Revision 1.6  2009/01/22 11:36:14  jsedwards
+-- Fix year in copyright.  NO code changes.
+--
+-- Revision 1.5  2009/01/19 14:08:04  jsedwards
+-- Changed so update can be called with 100% (1.0f) more than one time and
+-- verify it is never greater than 1.0f.  (Changes made on 2009-01-15)
+--
 -- Revision 1.4  2008/09/16 13:07:39  jsedwards
 -- Added new nwos_erase_progress_bar function.
 --
@@ -105,7 +112,8 @@ void nwos_start_progress_bar()
 
 void nwos_update_progress_bar(float percent)  /* 0.0 <= percent < 1.0 */
 {
-    assert(progress_bar_percent <= 100);
+    assert(percent <= 1.0f);
+    assert(progress_bar_percent <= 102);
 
     while (percent >= (float)progress_bar_percent / 100)
     {
