@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2004-2008  QRW Software
+--                 Copyright (C) 2004-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,18 @@
 --
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.39  2009/01/24 18:16:04  jsedwards
+-- Added test to nwos_remove_from_reference_list to verify the last_block
+-- pointer is valid after changing the reference list.  Also undefined
+-- VERIFY_LAST_BLOCK_POINTER so that verification is normally disabled.
+--
+-- Revision 1.38  2009/01/24 12:35:49  jsedwards
+-- Fix Bug #2392531 - Added new verify_last_block_pointer function and removed
+-- loop to find last block from nwos_add_to_reference_list.
+--
+-- Revision 1.37  2009/01/22 12:36:31  jsedwards
+-- Added last_block to cache.
+--
 -- Revision 1.36  2008/08/02 04:31:25  jsedwards
 -- Finished changes to read a reference list that has the old encryption at
 -- the beginning and later on has blocks with the new encryption.
@@ -179,6 +191,8 @@
 #include "objectify_private.h"
 
 
+#undef VERIFY_LAST_BLOCK_POINTER   /* disable tests that verify the last block pointer is correct */
+
 
 #define REF_LIST_CACHE_SIZE 64
 
@@ -186,6 +200,7 @@ typedef struct {
   ObjRef ref;
   int num_refs;
   Ref_List_First_Block* first_block;
+  Ref_List_Extra_Block* last_block;        /* null if only one block in list */
   int age;
 } Block_Cache;
 
@@ -240,6 +255,8 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
     cache->first_block->next_block_ptr = NULL;
 
+    cache->last_block = NULL;
+
     while (1)
     {
 	memset(ivec, 0, sizeof(ivec));
@@ -357,6 +374,8 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	prev_ptr = next_ptr;
 
+	cache->last_block = prev_ptr;        /* save it in cache in case this is the last block */
+
 	seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
     }
 }
@@ -381,6 +400,7 @@ static void write_reference_list(Block_Cache* cache)
     {
 	assert(cache->first_block->next_block_ptr == NULL);  /* the next block pointer should be null */
 	assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
+	assert(cache->last_block == NULL);
 
 	if (cache->first_block->list.common_header.flags == 0xffffffff)    /* block is dirty, needs to be written to disk */
 	{
@@ -495,6 +515,7 @@ static void write_reference_list(Block_Cache* cache)
 	}
 
 	assert(running_total == cache->num_refs);
+	assert(cache->last_block == prev_ptr);
     }
 
     if (block != NULL)    /* it needs to be written */
@@ -515,6 +536,7 @@ static void write_reference_list(Block_Cache* cache)
 
     /* just in case, nuke these */
     cache->first_block = NULL;
+    cache->last_block = NULL;
     memset(&cache->ref, 0, sizeof(cache->ref));
 }
 
@@ -687,6 +709,8 @@ void nwos_create_reference_list_with_existing_id(ObjRef* for_obj, ObjRef* ref_li
 
     memset(cache->first_block, 0, sizeof(Ref_List_First_Block));  /* zero it out */
 
+    cache->last_block = NULL;
+
     cache->num_refs = 0;
 
     memset(cache->first_block->ivec, 0, sizeof(cache->first_block->ivec));
@@ -736,6 +760,62 @@ void nwos_add_to_references(ObjRef* ref, ObjRef* obj)
 
 
 
+/* Verify the last_block pointer and offset are correct, only used in testing.  */
+
+#ifdef VERIFY_LAST_BLOCK_POINTER
+
+static void verify_last_block_pointer(Block_Cache* cache, int remaining, int calc_seq)
+{
+    int running_total;
+    int refs_remaining;
+    Ref_List_Extra_Block* next_ptr;
+    int seq;
+
+    assert(cache->num_refs > MAX_REFS_IN_REF_LIST);    /* there is more than one block */
+    assert(cache->first_block->next_block_ptr != NULL);         /* the next block pointer should not be null */
+    assert(cache->last_block != NULL);                          /* the last block pointer should not be null */
+    assert(cache->last_block->next_block_ptr == NULL);          /* but it's next block pointer should be */
+    assert(sizeof(ReferenceList) + MAX_REFS_IN_REF_LIST * sizeof(ObjRef) == FILE_BLOCK_SIZE - 4);
+
+    next_ptr = cache->first_block->next_block_ptr;        /* get pointer to first sidecar */
+    assert(is_same_object(&cache->first_block->next_block_ref, &next_ptr->id));  /* next pointer ref matches */
+
+    seq = 1;
+
+    running_total = MAX_REFS_IN_REF_LIST;
+
+    refs_remaining = cache->num_refs - running_total;
+
+    while (refs_remaining > MAX_REFS_IN_SIDECAR)    /* read more blocks */
+    {
+	assert(running_total < cache->num_refs);
+
+	assert(next_ptr->next_block_ptr != NULL);  /* the next block pointer should not be null */
+
+	/* next pointer ref matches */
+	assert(is_same_object(&next_ptr->next_block_ref, &next_ptr->next_block_ptr->id));
+
+	next_ptr = next_ptr->next_block_ptr;
+
+	seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+
+	running_total += MAX_REFS_IN_SIDECAR;
+
+	refs_remaining = cache->num_refs - running_total;
+    }
+
+    assert(next_ptr->next_block_ptr == NULL);  /* the next block pointer should be null */
+    assert(is_void_reference(&next_ptr->refs[refs_remaining]));
+
+    assert(next_ptr == cache->last_block);                         /* verify last_block is correct */
+    assert(running_total == ((cache->num_refs - MAX_REFS_IN_REF_LIST - 1) / MAX_REFS_IN_SIDECAR) * MAX_REFS_IN_SIDECAR + MAX_REFS_IN_REF_LIST);
+
+    assert(refs_remaining == remaining);
+    assert(seq == calc_seq);
+}
+#endif
+
+
 /* Add a new reference to the reference list specified (ref_list is the id of the reference list itself). */
 
 void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
@@ -747,6 +827,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
     int running_total;
     int refs_remaining;
     Ref_List_Extra_Block* next_ptr;
+    int block_num;
     uint8 ivec[IVEC_SIZE];
     int seq;
 
@@ -776,6 +857,7 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
     {
 	assert(cache->first_block->next_block_ptr == NULL);         /* the next block pointer should be null */
 	assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
+	assert(cache->last_block == NULL);
 
 	if (cache->num_refs < MAX_REFS_IN_REF_LIST)    /* there is room for more */
 	{
@@ -824,59 +906,66 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 	    memcpy(&next_ptr->refs[0], ref, sizeof(ObjRef));
 	    next_ptr->refs[1].word = 0;              /* void reference to mark the end of the list */
 
+	    cache->last_block = next_ptr;
+
 	    cache->num_refs++;
 	}
     }
     else
     {
 	assert(cache->first_block->next_block_ptr != NULL);         /* the next block pointer should not be null */
+	assert(cache->last_block != NULL);                          /* the last block pointer should not be null */
+	assert(cache->last_block->next_block_ptr == NULL);          /* but it's next block pointer should be */
 	assert(sizeof(ReferenceList) + MAX_REFS_IN_REF_LIST * sizeof(ObjRef) == FILE_BLOCK_SIZE - 4);
 
-	running_total = MAX_REFS_IN_REF_LIST;
-	next_ptr = cache->first_block->next_block_ptr;        /* get pointer to first sidecar */
+	next_ptr = cache->last_block;                         /* go right to the last block */
 
-	assert(is_same_object(&cache->first_block->next_block_ref, &next_ptr->id));  /* next pointer ref matches */
+	block_num = (cache->num_refs - MAX_REFS_IN_REF_LIST - 1) / MAX_REFS_IN_SIDECAR;
+	seq = (block_num + 1) % NUM_STORED_SEQ;
+	running_total = MAX_REFS_IN_REF_LIST + block_num * MAX_REFS_IN_SIDECAR;
 
-	seq = 1;
+	assert(running_total < cache->num_refs);
+	assert(cache->num_refs - running_total <= MAX_REFS_IN_SIDECAR);
 
-	while (running_total < cache->num_refs)    /* read more blocks */
+	refs_remaining = cache->num_refs - running_total;
+
+	assert(refs_remaining <= MAX_REFS_IN_SIDECAR);
+	assert(next_ptr->next_block_ptr == NULL);  /* the next block pointer should be null */
+	assert(is_void_reference(&next_ptr->refs[refs_remaining]));
+
+#ifdef VERIFY_LAST_BLOCK_POINTER
+	verify_last_block_pointer(cache, refs_remaining, seq);     /* verify last_block is correct */
+#endif
+
+	if (refs_remaining < MAX_REFS_IN_SIDECAR)    /* there is room for more */
 	{
-	    refs_remaining = cache->num_refs - running_total;
+	    memcpy(&next_ptr->refs[refs_remaining], ref, sizeof(ObjRef));
+	    next_ptr->refs[refs_remaining + 1].word = 0;  /* set the next ref to null */
+	    next_ptr->dirty = -1;                   /* mark it as dirty */
+	}
+	else       /* no room for more, must add new block */
+	{
+	    assert(refs_remaining == MAX_REFS_IN_SIDECAR);
 
-	    if (refs_remaining <= MAX_REFS_IN_SIDECAR)
-	    {
-		assert(next_ptr->next_block_ptr == NULL);  /* the next block pointer should be null */
-		assert(is_void_reference(&next_ptr->refs[refs_remaining]));
+	    nwos_generate_new_id(&next_ptr->next_block_ref);    /* id of new list */
 
-		if (refs_remaining < MAX_REFS_IN_SIDECAR)    /* there is room for more */
-		{
-		    memcpy(&next_ptr->refs[refs_remaining], ref, sizeof(ObjRef));
-		    next_ptr->refs[refs_remaining + 1].word = 0;  /* set the next ref to null */
-		    next_ptr->dirty = -1;                   /* mark it as dirty */
-		}
-		else       /* no room for more, must add new block */
-		{
-		    assert(refs_remaining == MAX_REFS_IN_SIDECAR);
+	    next_ptr->dirty = 0;   /* remove dirty flag */
 
-		    nwos_generate_new_id(&next_ptr->next_block_ref);    /* id of new list */
+	    nwos_crc32_calculate((uint8*) &next_ptr->refs, 
+				 (refs_remaining + 1) * sizeof(ObjRef),        /* include void ptr or next_block_ref */
+				 next_ptr->checksum);
 
-		    next_ptr->dirty = 0;   /* remove dirty flag */
+	    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
 
-		    nwos_crc32_calculate((uint8*) &next_ptr->refs, 
-					 (refs_remaining + 1) * sizeof(ObjRef),        /* include void ptr or next_block_ref */
-					 next_ptr->checksum);
+	    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
 
-		    memcpy(ivec, next_ptr->ivec, sizeof(ivec));
+	    next_ptr->next_block_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
 
-		    nwos_write_object_to_disk_and_encrypt(&next_ptr->id, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
+	    assert(next_ptr->next_block_ptr != NULL);
 
-		    next_ptr->next_block_ptr = nwos_malloc(sizeof(Ref_List_Extra_Block));
+	    next_ptr->next_block_ptr->next_block_ptr = NULL;
 
-		    assert(next_ptr->next_block_ptr != NULL);
-
-		    next_ptr->next_block_ptr->next_block_ptr = NULL;
-
-		    memcpy(next_ptr->next_block_ptr->ivec, ivec, sizeof(next_ptr->next_block_ptr->ivec));
+	    memcpy(next_ptr->next_block_ptr->ivec, ivec, sizeof(next_ptr->next_block_ptr->ivec));
 
 #ifdef LOG_REFERENCE_LIST
 	    snprintf(log_msg, sizeof(log_msg), "add_reference - next_block ref: %02x%02x%02x%02x obj: %p  next_block ref: %02x%02x%02x%02x obj: %p",
@@ -888,31 +977,17 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 	    nwos_log(log_msg);
 #endif
 
-		    next_ptr->next_block_ptr->dirty = -1;     /* mark it as dirty */
-		    memcpy(&next_ptr->next_block_ptr->id, &next_ptr->next_block_ref, sizeof(ObjRef));
-		    memcpy(&next_ptr->next_block_ptr->refs[0], ref, sizeof(ObjRef));
-		    next_ptr->next_block_ptr->refs[1].word = 0;              /* void reference to mark the end of the list */
-		}
+	    next_ptr->next_block_ptr->dirty = -1;     /* mark it as dirty */
+	    memcpy(&next_ptr->next_block_ptr->id, &next_ptr->next_block_ref, sizeof(ObjRef));
+	    memcpy(&next_ptr->next_block_ptr->refs[0], ref, sizeof(ObjRef));
+	    next_ptr->next_block_ptr->refs[1].word = 0;              /* void reference to mark the end of the list */
 
-		running_total += refs_remaining;
-	    }
-	    else
-	    {
-		assert(next_ptr->next_block_ptr != NULL);  /* the next block pointer should not be null */
-		running_total += MAX_REFS_IN_SIDECAR;
-
-		/* next pointer ref matches */
-		assert(is_same_object(&next_ptr->next_block_ref, &next_ptr->next_block_ptr->id));
-	    }
-
-	    next_ptr = next_ptr->next_block_ptr;
-
-	    seq = (seq + 1) % NUM_STORED_SEQ;    /* wrap around if we've used all of them */
+	    cache->last_block = next_ptr->next_block_ptr;
 	}
 
 	cache->num_refs++;
 
-	assert(running_total + 1 == cache->num_refs);
+	assert(running_total + refs_remaining + 1 == cache->num_refs);
     }
 }
 
@@ -989,6 +1064,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
     {
 	assert(cache->first_block->next_block_ptr == NULL);         /* the next block pointer should be null */
 	assert(is_void_reference(&cache->first_block->refs[cache->num_refs]));
+	assert(cache->last_block == NULL);
 
 	cache->num_refs--;
 
@@ -1016,6 +1092,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
     else   /* cases 2 through 8 - more than one block */
     {
 	assert(!is_void_reference(&cache->first_block->next_block_ref));
+	assert(cache->last_block != NULL);
 
 	for (i = 0; i < MAX_REFS_IN_REF_LIST; i++)
 	{
@@ -1046,10 +1123,13 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 	    if (cache->num_refs == (MAX_REFS_IN_REF_LIST + 1)) /* case 2 - don't need extra block anymore */
 	    {
+		assert(next_ptr == cache->last_block);
+
 		nwos_clear_bit_in_map(nwos_hash_ref(&cache->first_block->next_block_ref));
 		void_reference(&cache->first_block->next_block_ref);
 
 		cache->first_block->next_block_ptr = NULL;
+		cache->last_block = NULL;
 
 		nwos_free(next_ptr);
 		next_ptr = NULL;    /* flag that we don't need to move anymore */
@@ -1075,6 +1155,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 	}
 	else if (cache->num_refs == (MAX_REFS_IN_REF_LIST + 1))  /* case 4, two blocks with 1 entry in 2nd */
 	{
+	    assert(next_ptr == cache->last_block);
 	    assert(is_same_object(ref, &next_ptr->refs[0]));
 	    assert(is_void_reference(&next_ptr->refs[1]));
 
@@ -1082,6 +1163,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 	    void_reference(&cache->first_block->next_block_ref);
 
 	    cache->first_block->next_block_ptr = NULL;
+	    cache->last_block = NULL;
 
 	    nwos_free(next_ptr);
 	    next_ptr = NULL;    /* flag that we don't need to move anymore */
@@ -1123,6 +1205,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 	    if (i == 0 && is_void_reference(&next_ptr->refs[1]))  /* case 6 - only entry in this block */
 	    {
+		assert(next_ptr == cache->last_block);
 		assert(is_same_object(ref, &next_ptr->refs[0]));
 		assert(prev_ptr != NULL);
 		assert(prev_ptr->next_block_ptr == next_ptr);
@@ -1136,9 +1219,13 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 		nwos_free(next_ptr);
 
 		prev_ptr->dirty = -1;
+
+		cache->last_block = prev_ptr;
 	    }
 	    else if (next_ptr->next_block_ptr == NULL)   /* case 5 - just need to remove it from the last block */
 	    {
+		assert(next_ptr == cache->last_block);
+
 		while (!is_void_reference(&next_ptr->refs[i]))
 		{
 		    copy_reference(&next_ptr->refs[i], &next_ptr->refs[i+1]);
@@ -1183,6 +1270,7 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 
 		assert(prev_ptr->next_block_ptr == next_ptr);
 		assert(is_same_object(&prev_ptr->next_block_ref, &next_ptr->id));
+		assert(next_ptr == cache->last_block);
 
 		if (!is_void_reference(&next_ptr->refs[0]) && is_void_reference(&next_ptr->refs[1])) /* case 8 - only one entry in last block */
 		{
@@ -1195,6 +1283,8 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 		    prev_ptr->next_block_ptr = NULL;
 
 		    prev_ptr->dirty = -1;     /* mark it as dirty */
+
+		    cache->last_block = prev_ptr;
 		}
 		else  /* case 7 - just need to finish last block */
 		{
@@ -1215,6 +1305,29 @@ void nwos_remove_from_reference_list(ObjRef* ref, ObjRef* ref_list)
 	    }
 
 	    next_ptr = NULL;
+
+#ifdef VERIFY_LAST_BLOCK_POINTER
+	    if (cache->num_refs <= MAX_REFS_IN_REF_LIST)
+	    {
+		assert(cache->last_block == NULL);
+	    }
+	    else
+	    {
+		int block_num;
+		int running_total;
+		int refs_remaining;
+
+		assert(cache->last_block != NULL);
+
+		block_num = (cache->num_refs - MAX_REFS_IN_REF_LIST - 1) / MAX_REFS_IN_SIDECAR;
+		seq = (block_num + 1) % NUM_STORED_SEQ;
+		running_total = MAX_REFS_IN_REF_LIST + block_num * MAX_REFS_IN_SIDECAR;
+
+		refs_remaining = cache->num_refs - running_total;
+
+		verify_last_block_pointer(cache, refs_remaining, seq);     /* verify we didn't screw up the last_block pointer */
+	    }
+#endif
 	}
 
 	assert(next_ptr == NULL);
