@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2007-2008  QRW Software
+--                 Copyright (C) 2007-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,13 @@
 --
 --
 -- $Log: remove_file.c,v $
+-- Revision 1.14  2009/01/22 11:34:10  jsedwards
+-- Fixed year in copyright.  NO code changes.
+--
+-- Revision 1.13  2009/01/20 11:50:10  jsedwards
+-- Changed warning text about using this program and added test to see if
+-- backup (.dif) files are enabled and terminate execution if they are.
+--
 -- Revision 1.12  2008/09/06 15:34:23  jsedwards
 -- Changed to use TEST_ENVIRONMENT_VARIABLE.
 --
@@ -107,7 +114,7 @@ int main(int argc, char* argv[])
     int j;
     char name[256];
     char answer[8];
-
+    const char *backup_dir;
 
     if (argc != 3 || strcmp(argv[1], "--all-traces") != 0 || *argv[2] == '-')
     {
@@ -120,20 +127,36 @@ int main(int argc, char* argv[])
     if (getenv(TEST_ENVIRONMENT_VARIABLE) == NULL)
     {
 	printf("WARNING: while this program has been fairly thoroughly tested, it is possible\n");
-	printf("that it could damage your archive in some way.  I highly recommend making a\n");
-	printf("backup of your archive before using this program!\n\n");
+	printf("that it could damage your archive in some way.  It is also incompatible with\n");
+	printf("backup (.dif) files.  You must make a backup of your archive before using this\n");
+	printf("program and immediately after you use it!\n\n");
 
-	printf("Do you want to continue? (enter `yes' to remove %s) ", argv[2]);
+	backup_dir = nwos_get_backup_directory_path();
 
-	fflush(stdout);
-
-	fgets(answer, sizeof(answer), stdin);
-
-	printf("\n");
-
-	if (strcmp(answer, "yes\n") != 0)
+	if (backup_dir == NULL || *backup_dir == '\0')
 	{
-	    printf("No changes made!\n\n");
+	    printf("Have you made a backup of your archive? (enter `yes' to remove %s) ", argv[2]);
+
+	    fflush(stdout);
+
+	    fgets(answer, sizeof(answer), stdin);
+
+	    printf("\n");
+
+	    if (strcmp(answer, "yes\n") != 0)
+	    {
+		printf("No changes made!\n\n");
+		exit(1);
+	    }
+	}
+	else
+	{
+	    printf("Since you have backups (.dif files) enabled you will need to disable them\n");
+	    printf("before this program can be used.  One way of doing this is to set the\n");
+	    printf("OBJECTIFY_BACKUP_PATH environment variable to NULL, for example:\n\n");
+	    printf(" $ export OBJECTIFY_BACKUP_PATH=\n\n");
+	    printf("I repeat that, at this time, you should avoid using this program!\n\n");
+
 	    exit(1);
 	}
     }
