@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                   Copyright (C) 2008  QRW Software
+--                 Copyright (C) 2008-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,10 @@
 --
 --
 -- $Log: user_config.c,v $
+-- Revision 1.16  2009/01/02 21:44:29  jsedwards
+-- Added code to read the configuration before changing the private path so
+-- that it won't be overwritten.
+--
 -- Revision 1.15  2008/12/30 12:52:59  jsedwards
 -- Added set_private_objects_path function to allow changing the path for the
 -- sync_archive program.
@@ -358,6 +362,10 @@ void nwos_set_private_objects_path(const char* path)
 {
     int i;
 
+    if (strcmp(nwos_get_private_objects_path(), path) == 0) return;   /* If that is what it is already, no need to proceed */
+
+    /* NOTE: the above call also runs the normal stuff so it won't run after we set the private path and write over it. */
+
     for (i = 0; i < PATH_TABLE_SIZE; i++)
     {
 	if (strcmp(path_table[i].name, "private") == 0) break;
